c     Copyright (C)
c     2014-2018 Centre National de la Recherche Scientifique
c     2014-2018 Mesotar
c     
c     This file must be used under the terms of the CeCILL license.
c     This source file is licensed as described in the file COPYING, which
c     you should have received as part of this distribution.  The terms
c     are also available at
c     http://www.cecill.info/licences/Licence_CeCILL_V2.1-en.txt
c
      subroutine interpolate_spectra(Ns,Nk_ref,nu_ref,Nk,spectra,
     &     Nk_out,spectra_out,nu_hi_out)
      implicit none
      include 'max.inc'
c
c     Purpose: to interpolate every spectrum over a reference spectral discretization
c
c     Inputs:
c       + Ns: number of spectra
c       + Nk_ref: number of nu points in the "nu_ref" array
c       + nu_ref: reference spectral discretization
c       + Nk: number of points within each spectra
c       + spectra: the Ns nu/k(nu) spectra
c         - spectra(ik,1,i): wavenumber index 'ik' for spectra index 'i'
c         - spectra(ik,2,i): value of k(nu) index 'ik' for spectra index 'i'
c     
c     Outputs:
c       + Nk_out: number of points within each output spectra
c       + spectra_out: the Ns nu/k(nu) output spectra
c       + nu_hi_out: maximal wavenumber over output spectra
c     

c     I/O
      integer Ns
      integer Nk_ref
      double precision nu_ref(1:Nk_mx)
      integer Nk(1:Nfiles_mx)
      double precision spectra(1:Nk_mx,1:2,1:Nfiles_mx)
      integer Nk_out(1:Nfiles_mx)
      double precision spectra_out(1:Nk_mx,1:2,1:Nfiles_mx)
      double precision nu_hi_out
c     temp
      double precision nu_inf,nu_sup,nu_start,nu_end
      integer i,ik
      integer ok_inf,ok_sup
      integer ik1,ik1f,ik2,ik2f
      integer first_ik,last_ik
      integer j,js,jindex,jindexf
      integer ok,Nk_out0,truncation
c     Debug
      logical ik1_found,ik2_found
      double precision sum
      double precision ka_bar(1:Nfiles_mx)
c     Debug
c     label
      integer strlen
      character*(Nchar_mx) label
      label='subroutine interpolate_spectra'

c     small verifications:
c     1) check every spectra has the same limits the others have
      nu_inf=spectra(1,1,1)
      nu_sup=spectra(Nk(1),1,1)
      ok_inf=1
      ok_sup=1
      do i=1,Ns
         if (spectra(1,1,i).ne.nu_inf) then
            ok_inf=0
         endif
         if (spectra(Nk(i),1,i).ne.nu_sup) then
            ok_sup=0
         endif
      enddo ! i
      if (ok_inf.eq.0) then
         call error(label)
         write(*,*) 'All spectra do not start at the same nu'
         stop
      endif
      if (ok_sup.eq.0) then
         call error(label)
         write(*,*) 'All spectra do not stop at the same nu'
         stop
      endif
c     2) check where these limits are
c     ik1 is the index of the first wavenumber in the nu_ref array
c     that is greater or equal to nu_inf
      if (nu_inf.le.nu_ref(1)) then
         ik1=1
         ik1f=1
         nu_start=nu_ref(1)
      else
         nu_start=nu_inf
         ik1f=0
         do ik=2,Nk_ref
            if ((nu_ref(ik-1).lt.nu_inf).and.
     &           (nu_ref(ik).ge.nu_inf)) then
               ik1f=1
               ik1=ik
               goto 111
            endif
         enddo !ik
      endif
 111  continue
      if (ik1f.eq.0) then
         call error(label)
         write(*,*) 'ik1 could not be found'
         stop
      endif
c     Debug
c      write(*,*) 'nu_start=',nu_start
c     Debug
c     ik2 is the index of the last wavenumber in the nu_ref array
c     that is lower or equal to nu_sup
      if (nu_sup.ge.nu_ref(Nk_ref)) then
         ik2=Nk_ref
         ik2f=1
         nu_end=nu_ref(Nk_ref)
      else
         nu_end=nu_sup
         ik2f=0
         do ik=2,Nk_ref
            if ((nu_ref(ik-1).lt.nu_sup).and.
     &           (nu_ref(ik).ge.nu_sup)) then
               ik2f=1
               ik2=ik-1
               goto 222
            endif
         enddo !ik
      endif
 222  continue
      if (ik2f.eq.0) then
         call error(label)
         write(*,*) 'ik2 could not be found'
         stop
      endif
c     Debug
c      write(*,*) 'nu_end=',nu_end
c     Debug

c     limits of the indexes the values of k have to be interpolated for
      if (nu_ref(ik1).eq.nu_inf) then
         first_ik=ik1+1
      else
         first_ik=ik1
      endif
      if (nu_ref(ik2).eq.nu_sup) then
         last_ik=ik2-1
      else
         last_ik=ik2
      endif
c     Nk_out0 is the number of values in output arrays
      Nk_out0=last_ik-first_ik+3
      if (Nk_out0.gt.Nk_mx) then ! need to truncate
         truncation=1
      else
         truncation=0
      endif
c     If truncation is required...
      if (truncation.eq.1) then
c     find the index of the last nu_ref(ik2)
         last_ik=Nk_mx+first_ik-2
         Nk_out0=Nk_mx
         nu_hi_out=nu_ref(last_ik)
      else
         nu_hi_out=nu_end
      endif
c     Debug
c      write(*,*) 'truncation=',truncation,nu_hi_out
c     Debug

c     Debug
c      write(*,*) 'ik1=',ik1,' ik2=',ik2
c      write(*,*) 'nu_ref(',ik1,')=',nu_ref(ik1)
c      write(*,*) 'nu_ref(',ik2-1,')=',nu_ref(ik2-1)
c      write(*,*) 'nu_ref(',ik2,')=',nu_ref(ik2)
c      write(*,*) 'first_ik=',first_ik,' last_ik=',last_ik
c     Debug
      
c     Interpolate each spectra
      do i=1,Ns
c     initialization
         Nk_out(i)=0
c     first nu=nu_inf
         js=2
         jindexf=0
         do j=js,Nk(i)
            if ((nu_start.ge.spectra(j-1,1,i)).and.
     &           (nu_start.le.spectra(j,1,i))) then
               jindexf=1
               jindex=j
               goto 112
            endif
         enddo                  ! j
 112     continue
         if (jindexf.eq.0) then
            call error(label)
            write(*,*) 'nu_start=',nu_start
            write(*,*) 'could not be located within spectra index',i
            write(*,*) 'nu_inf=',nu_inf
            write(*,*) 'nu_sup=',nu_sup
            stop
         else
            js=jindex
         endif
         Nk_out(i)=Nk_out(i)+1
         spectra_out(Nk_out(i),1,i)=nu_start
c     first k
         call linear_interpolation(spectra(jindex-1,1,i),
     &        spectra(jindex,1,i),spectra(jindex-1,2,i),
     &        spectra(jindex,2,i),nu_start,
     &        spectra_out(Nk_out(i),2,i))
c         spectra_out(Nk_out(i),2,i)=spectra(1,2,i)
         do ik=first_ik,last_ik
c     looking for position of "nu_ref(ik)" in the current spectra
            jindexf=0
            do j=js,Nk(i)
c            do j=2,Nk(i)
               if ((nu_ref(ik).ge.spectra(j-1,1,i)).and.
     &              (nu_ref(ik).le.spectra(j,1,i))) then
                  jindexf=1
                  jindex=j
                  goto 333
               endif
            enddo ! j
 333        continue
            if (jindexf.eq.0) then
               call error(label)
               write(*,*) 'nu_ref(',ik,')=',nu_ref(ik)
               write(*,*) 'nu_ref(',ik-1,')=',nu_ref(ik-1)
               write(*,*) 'could not be located within spectra index',i
               write(*,*) 'nu_inf=',nu_inf
               write(*,*) 'nu_sup=',nu_sup
               write(*,*) 'spectra(',js-2,')=',spectra(js-2,1,i)
               write(*,*) 'spectra(',js-1,')=',spectra(js-1,1,i)
               write(*,*) 'spectra(',Nk(i),')=',spectra(Nk(i),1,i)
               stop
            else
               js=jindex
            endif
c     number of elements
            Nk_out(i)=Nk_out(i)+1
            if (Nk_out(i).gt.Nk_mx) then
               call error(label)
               write(*,*) 'Nk_out(',i,')=',Nk_out(i)
               write(*,*) 'while Nk_mx=',Nk_mx
               stop
            endif
c     value of nu
            spectra_out(Nk_out(i),1,i)=nu_ref(ik)
c     value of k(nu)
            call linear_interpolation(spectra(jindex-1,1,i),
     &           spectra(jindex,1,i),spectra(jindex-1,2,i),
     &           spectra(jindex,2,i),nu_ref(ik),
     &           spectra_out(Nk_out(i),2,i))
         enddo ! ik
         if ((truncation.eq.0).and.
     &        (spectra_out(Nk_out(i),1,i).ne.nu_end)) then
            jindexf=0
            do j=js,Nk(i)
               if ((nu_end.gt.spectra(j-1,1,i)).and.
     &              (nu_end.le.spectra(j,1,i))) then
                  jindexf=1
                  jindex=j
                  goto 777
               endif
            enddo ! j
 777        continue
            if (jindexf.eq.0) then
               call error(label)
               write(*,*) 'nu_end=',nu_end
               write(*,*) 'could not be located within spectra index',i
               write(*,*) 'nu_inf=',nu_inf
               write(*,*) 'nu_sup=',nu_sup
               stop
            endif
            Nk_out(i)=Nk_out(i)+1
            if (Nk_out(i).gt.Nk_mx) then
               call error(label)
               write(*,*) 'Nk_out(',i,') final=',Nk_out(i)
               write(*,*) 'while Nk_mx=',Nk_mx
               stop
            endif
c     last k
            spectra_out(Nk_out(i),1,i)=nu_end
            call linear_interpolation(spectra(jindex-1,1,i),
     &           spectra(jindex,1,i),spectra(jindex-1,2,i),
     &           spectra(jindex,2,i),nu_end,
     &           spectra_out(Nk_out(i),2,i))
c            spectra_out(Nk_out(i),2,i)=spectra(Nk(i),2,i)
         endif
      enddo ! i

c     Debug
c      write(*,*) Nk_out(1)-1,spectra(Nk_out(1)-1,1,1)
c      write(*,*) Nk_out(1),spectra(Nk_out(1),1,1)
c     Debug

c     small verifications
      ok=1
      do i=1,Ns
         if (Nk_out(i).ne.Nk_out(1)) then
            ok=0
            goto 444
         endif
      enddo ! i
 444  continue
      if (ok.eq.0) then
         call error(label)
         write(*,*) 'Interpolated spectra have not the same size'
         do i=1,Ns
            write(*,*) 'Nk_out(',i,')=',Nk_out(i)
         enddo
         stop
      endif

      ok=1
      do i=1,Ns
         if (spectra_out(Nk_out(i),1,i).ne.nu_hi_out) then
            ok=0
            goto 555
         endif
      enddo ! i
 555  continue
      if (ok.eq.0) then
         call error(label)
         write(*,*) 'Interpolated spectra have not same ends'
         stop
      endif
c$$$
c$$$c     computation of ka_bar in a given spectral interval
c$$$      nu_inf=4000.0D+0
c$$$      nu_sup=4020.0D+0
c$$$c     identify 'ik1', the location of 'nu_inf' in the spectral mesh
c$$$      ik1_found=.false.
c$$$      do ik=1,Nk_out(1)
c$$$         if ((.not.ik1_found).and.(spectra_out(ik,1,1).ge.nu_inf)) then
c$$$            ik1=ik
c$$$            ik1_found=.true.
c$$$            goto 412
c$$$         endif
c$$$      enddo ! ik
c$$$ 412  continue
c$$$      if (.not.ik1_found) then
c$$$         call error(label)
c$$$         write(*,*) 'ik1 could not be found'
c$$$         write(*,*) 'nu_inf=',nu_inf
c$$$         write(*,*) 'spectra_out(1)='
c$$$     &        ,spectra_out(1,1,1)
c$$$         write(*,*) 'spectra_out(',Nk_out(1),')='
c$$$     &        ,spectra_out(Nk_out(1),1,1)
c$$$         stop
c$$$      endif
c$$$c     identify 'ik2', the location of 'nu_sup' in the spectral mesh
c$$$      ik2_found=.false.
c$$$      do ik=ik1,Nk_out(1)
c$$$         if ((.not.ik2_found).and.(spectra_out(ik,1,1).ge.nu_sup)) then
c$$$            ik2=ik
c$$$            ik2_found=.true.
c$$$            goto 413
c$$$         endif
c$$$      enddo ! ik
c$$$ 413  continue
c$$$      if (.not.ik2_found) then
c$$$         call error(label)
c$$$         write(*,*) 'ik2 could not be found'
c$$$         write(*,*) 'nu_sup=',nu_sup
c$$$         write(*,*) 'spectra_out(1)='
c$$$     &        ,spectra_out(1,1,1)
c$$$         write(*,*) 'spectra_out(',Nk_out(1),')='
c$$$     &        ,spectra_out(Nk_out(1),1,1)
c$$$         stop
c$$$      endif
c$$$c     Average ka in the band
c$$$      do i=1,Ns
c$$$         sum=0.0D+0
c$$$         do ik=ik1,ik2-1
c$$$            sum=sum
c$$$     &           +(spectra_out(ik,2,i)+spectra_out(ik+1,2,i))
c$$$     &           *(spectra_out(ik+1,1,i)-spectra_out(ik,1,i))
c$$$         enddo ! ik
c$$$         ka_bar(i)=sum
c$$$     &        /(2.0D+0*(spectra_out(ik2,1,1)-spectra_out(ik1,1,1)))
c$$$      enddo ! i
c$$$
c$$$      open(20,file='./results/ka_bar_profile.txt')
c$$$      do i=1,Ns
c$$$         write(20,*) i,ka_bar(i)
c$$$      enddo ! i
c$$$      close(20)

      return
      end
